package controllers_pages

import (
	"main/mvc/models"
	"main/mvc/models/models_pages"
	"net/http"
	"path"
	"strconv"
)

// Обработчик странички со списком постов
func PostsPageHandler(app *models.App) http.HandlerFunc {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		var err error

		// Ссылка вида /posts/0
		link := r.URL.Path

		// Страничка рендерится только если ее нет в кэше
		pageData, ok := app.PagesCache.Get(link)
		if !ok {

			// Из мапы формирую отсортированный список всех постов (сначала новые)
			postsList := app.PostsMap.PostsList()

			// Ошибки тут быть не может, так как этот обработчик настроен
			// на только существующие реально pageNumber и в случае
			// какого то не существующего номера страницы он просто
			// не сработает (в главном main настроены маршруты)
			pageNumber, _ := strconv.Atoi(path.Base(link))

			page := models_pages.CreatePage(postsList, pageNumber, app.Cfg.PostsMaxCountOnPage)

			pageData, err = page.RenderPostsPage(app.Templates, app.Version)
			if err != nil {
				http.Error(w, err.Error(), http.StatusInternalServerError)
				return
			}
			app.PagesCache.Set(link, pageData)
		}

		sendPostsPage(w, pageData)
	})
}

// Отправляет страницу
func sendPostsPage(w http.ResponseWriter, data []byte) {
	w.Header().Set("Content-Type", "text/html; charset=utf-8")
	w.WriteHeader(http.StatusOK)
	w.Write(data)
}
